package com.ningdatech.pmapi.scheduler.task;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.http.HttpStatus;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.ningdatech.basic.model.ApiResponse;
import com.ningdatech.pmapi.common.statemachine.util.StateMachineUtils;
import com.ningdatech.pmapi.projectlib.enumeration.ProjectStatusEnum;
import com.ningdatech.pmapi.projectlib.model.entity.Project;
import com.ningdatech.pmapi.projectlib.service.IProjectService;
import com.ningdatech.pmapi.provincial.enumeration.ProjectProvincialAuditStatusEnum;
import com.ningdatech.pmapi.provincial.model.res.ProvincialProjectRes;
import com.ningdatech.pmapi.provincial.service.IJoinReviewProvincialBureauService;
import com.ningdatech.pmapi.scheduler.contants.TaskContant;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.time.LocalDateTime;
import java.util.List;
import java.util.Objects;

/**
 * @Classname CheckProvincialReviewResultTask
 * @Description 去获取省级联审结果任务
 * @Date 2023/3/16 10:12
 * @Author PoffyZhang
 */
@Slf4j
@Component
@RequiredArgsConstructor
public class CheckProvincialReviewResultTask {

    private final IJoinReviewProvincialBureauService reviewProvincialBureauService;

    private final IProjectService projectService;
    private final StateMachineUtils stateMachineUtils;

    @Scheduled(cron = "0 */1 * * * ?")
    public void statusFlow() throws UnknownHostException {
        //测试暂时用自己电脑HOST
        if (TaskContant.Host.HOST_ZPF.equals(InetAddress.getLocalHost().getHostName())) {
            //1. 定时取 省级部门联审中的项目 去取项目
            List<Project> projectList = projectService.list(Wrappers.lambdaQuery(Project.class)
                            .eq(Project::getStage, ProjectStatusEnum.NOT_APPROVED.getCode())
                            .eq(Project::getStatus, ProjectStatusEnum.JOINT_REVIEW_BY_PROVINCIAL_DEPARTMENTS.getCode())
                            .orderByAsc(Project::getCreateOn));

            log.info("需要去查询省级联审结果的项目 size:{}",projectList.size());
            if(CollUtil.isEmpty(projectList)){
                log.info("没有正在省级联审中的项目!");
                return;
            }

            //遍历
            for(Project project: projectList){
                try{
                    ApiResponse apiResponse = reviewProvincialBureauService.processInfo(String.valueOf(project.getId()));
                    log.info("项目 【{}】 去获取省局联审结果 :{}",project.getId(),apiResponse);
                    if(Objects.isNull(apiResponse) || !Integer.valueOf(HttpStatus.HTTP_OK).equals(apiResponse.getCode())){
                        log.info("项目 【{}】 去获取省局联审结果失败",project.getId());
                        continue;
                    }
                    ProvincialProjectRes projectRes = JSON.parseObject(JSON.toJSONString(apiResponse.getData()),
                            ProvincialProjectRes.class);

                    if(ProjectProvincialAuditStatusEnum.AUDITING.getCode().equals(projectRes.getProjectStatus())){
                        log.info("此项目 【{}】 还在审核中",projectRes.getProjectId());
                    }else if(ProjectProvincialAuditStatusEnum.SUCCESS.getCode().equals(projectRes.getProjectStatus())){
                        log.info("此项目 【{}】 审核通过",projectRes.getProjectId());
                        stateMachineUtils.pass(project);
                        project.setUpdateOn(LocalDateTime.now());
                        projectService.updateById(project);
                    }else if(ProjectProvincialAuditStatusEnum.SUCCESS.getCode().equals(projectRes.getProjectStatus())){
                        log.info("此项目 【{}】 审核不通过",projectRes.getProjectId());
                        stateMachineUtils.reject(project);
                        project.setUpdateOn(LocalDateTime.now());
                        projectService.updateById(project);
                    }else{
                        log.info("此项目 【{}】 审核结果错误",projectRes.getProjectId());
                    }
                }catch (Exception e){
                    log.error("项目审核信息获取异常 projectId:【" + project.getId() + "】 异常内容:" + e.getMessage());
                }
            }
        }
    }
}
